/****************************************************************************

Author : Dario Di Turi
Version : 1.0
Date : 25/01/2013

CopyRight 2013 all rights are reserved


********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Dario Di Turi (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.



*/



#include <p18f4550.h>
#include <portb.h>

#include "delay.h"
#include "delay.c"

#include "AD9833.h"
#include "AD9833.c"

#define   LCD_DEFAULT
#include "LCD_44780.h"
#include "LCD_44780.c"

#pragma config FOSC = HS 	
#pragma config WDT = OFF 	
#pragma config LVP = OFF 	
#pragma config PBADEN = OFF 

//OSC = HS  	Impostato per lavorare ad alta frequenza 
//WDT = OFF     Disabilito il watchdog timer
//LVP = OFF 	Disabilito programmazione LVP
//PBADEN = OFF  Disabilito gli ingressi analogici

//************************************************************
// DICHIARAZIONI NOMI MNEMONICI
//************************************************************
#define BT_1     PORTBbits.RB4
#define BT_2     PORTBbits.RB5
#define BT_3     PORTBbits.RB6
#define BT_4     PORTBbits.RB7

#define GRADI 0xDF

//************************************************************
// PROTOTIPI FUNZIONI
//************************************************************
unsigned int cambia_incremento_freq(void);		   // Cambio il valore di incremento della frequenza
void cambia_uscita(void);						   // Cambia forma d'onda uscita
void incremento_frequenza(unsigned int incr);      // Incremento la frequenza
void decremento_frequenza(unsigned int decr);      // Decremento la frequenza
void incremento_fase(void);						   // Incremento la fase
void decremento_fase(void);						   // Decremento la fase
void scrivi_frequenza_display(unsigned long freq); // Scrivo la frequenza sul display

//************************************************************
// DICHIARAZIONI VARIABILI
//************************************************************

unsigned long    frequency = 1000;
unsigned short   phase = 0;
signed char      tuning=1;
signed char      mode=1;
unsigned int     incremento=1;
 


void main (void){
	 
	// Imposto PORTA tutti ingressi meno RC0 e RA5(FSYNC)
	LATA = 0x00; 
 	TRISA = 0b11011110;

	// Imposto PORTB tutti ingressi meno RB1(SCLK)
 	LATB = 0x00; 
 	TRISB = 0b11111101;

	// Imposto PORTC tutti ingressi meno RC0, RC1 e RC7(SDATA)
 	LATC = 0x00;    	 
 	TRISC = 0b01111100;
 	
	// Imposto PORTD tutte uscite, RD0 come ingresso
	LATD = 0x00;
	TRISD = 0b00000001;

	// Imposto PORTE tutti ingressi
 	LATE = 0x00;
 	TRISE = 0xFF;
	
	// Imposto il quarzo per la libreria delay
	setQuartz (20);
	
	// Abilita i resistori di pull-up sulla PORTB
	EnablePullups();
		 
	// Inizializzo il display LCD con quarzo a 20MHz
	OpenLCD (20);
	BacklightLCD (TURN_ON_LED);
	WriteStringLCD ("F=           Hz");
  	Line2LCD ();
	WriteStringLCD (":  0");
	WriteCharLCD(GRADI);
	WriteStringLCD (" :SIN  :1  ");
	ShiftCursorLCD (LEFT,1);
	WriteIntLCD (phase,3);
	
 
	// Inizializzo il dispositivo AD9833 con MCLK a 8MHz, forma sinusoidale, FREQ0 out	
	init_AD9833(25);
	set_frequency_AD9833(frequency,0);
	scrivi_frequenza_display(frequency);
		
	// Ciclo infinito
 	while (1) {

                if (BT_1 == 0 && BT_4 == 0) {
			delay_ms(150);
			if ((BT_1 == 0 && BT_4 == 0)){
				incremento_fase();
			}	
		}
		
		if (BT_2 == 0 && BT_4 == 0) {
			delay_ms(150);
			if (BT_2 == 0 && BT_4 == 0){
				decremento_fase();
			}	
		}
                
		if (BT_3 == 0 && BT_4 == 0){
			delay_ms(150);
			if (BT_3 == 0 && BT_4 == 0){
				incremento=cambia_incremento_freq();
			}
		}

		if (BT_3 == 0 && BT_4 == 1){
			delay_ms(150);
			if (BT_3 == 0 && BT_4 == 1){
				cambia_uscita();
			}
		}

		if (BT_1 == 0 && BT_4 == 1){
			delay_ms(150);
			if (BT_1 == 0 && BT_4 == 1){
				incremento_frequenza(incremento);
			}	
			
		}
			  
		if (BT_2 == 0 && BT_4 == 1){
			delay_ms(150);
			if (BT_2 == 0 && BT_4 == 1){
				decremento_frequenza(incremento);
			}	
		}
	
	}
}


//************************************************************
// FUNZIONI
//************************************************************


// Cambio il valore di incremento della frequenza
unsigned int cambia_incremento_freq(void){

	unsigned int increase;
	
	switch (tuning){
	
		case 0:
                        increase=0;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("FI ");
                        return(increase);
                        break;
				
		case 1:
                        increase=1;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("1  ");
                        return(increase);
                        break;

		case 2:
                        increase=10;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("10 ");
                        return(increase);
                        break;

		case 3:
                        increase=100;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("100");
                        return(increase);
                        break;

		case 4:
                        increase=1000;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("1K ");
                        return(increase);
                        break;

		case 5:
                        increase=10000;
                        tuning++;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("10K");
                        return(increase);
                        break;
		
		case 6:
                        increase=50000;
                        tuning=0;
                        Line2LCD ();
                        ShiftCursorLCD (RIGHT,13);
                        WriteStringLCD ("50K");
                        return(increase);
                        break;
		
	}

}

// Incremento la frequenza
void incremento_frequenza(unsigned int incr){
	
	if(incr==0){
		tuning_frequency_AD9833(1,UP,REG0);
	}
	else{
		frequency = frequency+incr;
		if (frequency > 12500000){
			frequency = 12500000;
		}
		set_frequency_AD9833(frequency,0);
		scrivi_frequenza_display(frequency);
	}
}

// Decremento la frequenza
void decremento_frequenza(unsigned int decr){
	
	if(decr==0){
		tuning_frequency_AD9833(1,DOWN,REG0);
	}
	else{
		if(frequency < decr){
			frequency = 0;
		}
		else{
			frequency = frequency - decr;
		}
		set_frequency_AD9833(frequency,0);
		scrivi_frequenza_display(frequency);
	}
}

// Incremento la fase
void incremento_fase(void){

	phase=phase+100;
		if(phase>=3600){
			phase=0;
		}
		set_phase_AD9833(phase,REG0);
		Line2LCD ();
		ShiftCursorLCD (RIGHT,1);
		WriteIntLCD (phase/10,3);
}

// Decremento la fase
void decremento_fase(void){

	
		if(phase<=100){
			phase=3600;
		}
		else{
			phase=phase-100;
		}
		set_phase_AD9833(phase,REG0);
		Line2LCD ();
		ShiftCursorLCD (RIGHT,1);
		WriteIntLCD (phase/10,3);
}

// Scrivo la frequenza sul display
void scrivi_frequenza_display(unsigned long freq){

    unsigned char numero_cifre = 0;
	unsigned long x;
	unsigned char Stringa[8];
	
	//Conto il numero di caratteri presenti nella cifra dividendo il numero per 10 
	for (x = freq; x > 0; x /= 10){
		numero_cifre++;
	}
	
	//Preparo il display per la scrittura
	HomeLCD();
	ShiftCursorLCD (RIGHT,4);
	WriteStringLCD ("        ");
	ShiftCursorLCD (LEFT,numero_cifre);
	
	//Aggiungo il carattere di fine stringa all'ultima posizione
	Stringa[numero_cifre] = '\0';

	//Faccio modulo di x % 10 e aggiungo '0' per ottenere il carattere ASCII
	for (x = freq; x > 0; x /= 10){
	    Stringa[--numero_cifre] = (x % 10) + '0'; 
	}
	
	WriteVarLCD(Stringa);
	
}

//Cambia forma d'onda uscita
void cambia_uscita(void){

	mode++;
		
	switch (mode){
	
		case 1:
                    set_function_AD9833(SIN);
                    Line2LCD ();
                    ShiftCursorLCD (RIGHT,7);
                    WriteStringLCD ("SIN");
                    break;
		
		case 2:
                    set_function_AD9833(TRIANGLE);
                    Line2LCD ();
                    ShiftCursorLCD (RIGHT,7);
                    WriteStringLCD ("TRI");
                    break;
		
		case 3:
                    set_function_AD9833(SQUARE);
                    Line2LCD ();
                    ShiftCursorLCD (RIGHT,7);
                    WriteStringLCD ("SQU");
                    mode=0;
                    break;
	}	

}
